# Generated by Django 4.2.16 on 2024-10-19 05:07

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("slack_messages", "0003_default_recipient"),
    ]

    operations = [
        migrations.AddField(
            model_name="slackmessagingpolicy",
            name="template_type",
            field=models.CharField(
                choices=[("D", "Dictionary"), ("DJ", "Django"), ("DI", "Django Inline"), ("?", "Unknown")],
                default="D",
                help_text="Type of message template.",
                max_length=2,
                verbose_name="Template type",
            ),
        ),
    ]
